@@ -27,96 +27,107 @@ function(model, ...)
   dataset <- model@Dataset
 
   if (missing(dataset) || (length(dataset) == 0)) {
-    dataset <- as.matrix(model@x@Dataset[[model@pos]])
+    dataset <- model@x@Dataset[[model@pos]]
   }
-  else {
+    
+  if (as.character(class(dataset)) == "data.frame") {
+    Y.type <- 0
+    
     dataset <- as.matrix(dataset)
-  }  
+    
+    n <- nrow(dataset)
 
-  n <- nrow(dataset)
+    if (sum(pdf %in% .rebmix$pdf[c(4, 6)]) == c * d) {
+      model@p <- w
 
-  if (sum(pdf %in% .rebmix$pdf[c(4, 6)]) == c * d) {
-    model@p <- w
+      model@pi <- list(); nlevels <- array()
 
-    model@pi <- list(); nlevels <- array()
+      for (i in 1:d) {
+        for (j in 1:c) {
+          if (pdf[(j - 1) * d + i] == .rebmix$pdf[4]) {
+            if (j == 1) {
+              nlevels[i] <- as.integer(theta1[(j - 1) * d + i]) + 1
 
-    for (i in 1:d) {
-      for (j in 1:c) {
-        if (pdf[(j - 1) * d + i] == .rebmix$pdf[4]) {
-          if (j == 1) {
-            nlevels[i] <- as.integer(theta1[(j - 1) * d + i]) + 1
+              model@pi[[i]] <- matrix(data = 0.0, nrow = c, ncol = nlevels[i])
 
-            model@pi[[i]] <- matrix(data = 0.0, nrow = c, ncol = nlevels[i])
+              colnames(model@pi[[i]]) <- paste(0:(nlevels[i] - 1), sep = "")
+              rownames(model@pi[[i]]) <- paste(1:c, sep = "")
+            }
 
-            colnames(model@pi[[i]]) <- paste(0:(nlevels[i] - 1), sep = "")
-            rownames(model@pi[[i]]) <- paste(1:c, sep = "")
+            for (ii in 1:nlevels[i]) {
+              model@pi[[i]][j, ii] <- dbinom(ii - 1, size = as.integer(theta1[(j - 1) * d + i]), prob = as.numeric(theta2[(j - 1) * d + i]))
+            }
           }
+          else
+          if (pdf[(j - 1) * d + i] == .rebmix$pdf[6]) {
+            if (j == 1) {
+              nlevels[i] <- length(unique(dataset[, i]))
 
-          for (ii in 1:nlevels[i]) {
-            model@pi[[i]][j, ii] <- dbinom(ii - 1, size = as.integer(theta1[(j - 1) * d + i]), prob = as.numeric(theta2[(j - 1) * d + i]))
-          }
-        }
-        else
-        if (pdf[(j - 1) * d + i] == .rebmix$pdf[6]) {
-          if (j == 1) {
-            nlevels[i] <- length(unique(dataset[, i]))
+              model@pi[[i]] <- matrix(data = 0.0, nrow = c, ncol = nlevels[i])
 
-            model@pi[[i]] <- matrix(data = 0.0, nrow = c, ncol = nlevels[i])
+              colnames(model@pi[[i]]) <- paste(0:(nlevels[i] - 1), sep = "")
+              rownames(model@pi[[i]]) <- paste(1:c, sep = "")
+            }
 
-            colnames(model@pi[[i]]) <- paste(0:(nlevels[i] - 1), sep = "")
-            rownames(model@pi[[i]]) <- paste(1:c, sep = "")
-          }
-
-          for (ii in 1:nlevels[i]) {
-            model@pi[[i]][j, ii] <- ddirac(ii - 1, location = as.integer(theta1[(j - 1) * d + i]))
-          }
-        }
-      }
-    }
-
-    Y <- dataset; y <- as.matrix(unique(Y)); Nt <- array(); Np <- array()
-
-    for (j in 1:nrow(y)) {
-      x <- array(); k <- 1
-
-      for (l in 1:nrow(Y)) {
-        if (all(y[j, ] == Y[l, ])) {
-          x[k] <- l; k <- k + 1
-        }
-      }
-
-      Nt[j] <- length(x); Y <- as.matrix(Y[-x, ]); Np[j] <- 0.0
-
-      for (l in 1:c) {
-        Pl <- 1.0
-
-        for(i in 1:d) {
-          for (ii in 1:length(model@pi[[i]][l, ])) {
-            if (y[j, i] == ii - 1) {
-              Pl <- Pl * model@pi[[i]][l, ii]
+            for (ii in 1:nlevels[i]) {
+              model@pi[[i]][j, ii] <- ddirac(ii - 1, location = as.integer(theta1[(j - 1) * d + i]))
             }
           }
         }
-
-        Np[j] <- Np[j] + model@p[l] * Pl * n
       }
-    }
 
-    model@P <- as.data.frame(cbind(y, Nt, Np))
+      Y <- dataset; y <- as.matrix(unique(Y)); Nt <- array(); Np <- array()
 
-    if (is.null(colnames(dataset))) {
-      colnames(model@P) <- paste(c(1:d, "Nt", "Np"), sep = "")
+      for (j in 1:nrow(y)) {
+        x <- array(); k <- 1
+
+        for (l in 1:nrow(Y)) {
+          if (all(y[j, ] == Y[l, ])) {
+            x[k] <- l; k <- k + 1
+          }
+        }
+
+        Nt[j] <- length(x); Y <- as.matrix(Y[-x, ]); Np[j] <- 0.0
+
+        for (l in 1:c) {
+          Pl <- 1.0
+
+          for(i in 1:d) {
+            for (ii in 1:length(model@pi[[i]][l, ])) {
+              if (y[j, i] == ii - 1) {
+                Pl <- Pl * model@pi[[i]][l, ii]
+              }
+            }
+          }
+
+          Np[j] <- Np[j] + model@p[l] * Pl * n
+        }
+      }
+
+      model@P <- as.data.frame(cbind(y, Nt, Np))
+
+      if (is.null(colnames(dataset))) {
+        colnames(model@P) <- paste(c(1:d, "Nt", "Np"), sep = "")
       
-      names(model@pi) <- 1:d
-    }
-    else {
-      colnames(model@P) <- c(colnames(dataset), "Nt", "Np")
+        names(model@pi) <- 1:d
+      }
+      else {
+        colnames(model@P) <- c(colnames(dataset), "Nt", "Np")
       
-      names(model@pi) <- colnames(dataset)
-    }
+        names(model@pi) <- colnames(dataset)
+      }
 
-    rownames(model@pi[[i]]) <- paste(1:c, sep = "")
-  }
+      rownames(model@pi[[i]]) <- paste(1:c, sep = "")
+    }
+  }  
+  else
+  if (as.character(class(dataset)) == "Histogram") {
+    Y.type <- 1
+    
+    dataset <- as.matrix(dataset@Y)
+    
+    n <- nrow(dataset)
+  }   
 
   output <- .C(C_RCombineComponentsMIX,
     c = as.integer(c),
@@ -127,7 +138,8 @@ function(model, ...)
     pdf = as.character(pdf),
     Theta = as.double(c(theta1, theta2, theta3)),
     n = as.integer(n),
-    x = as.double(dataset),
+    Y = as.double(dataset),
+    Y.type = as.integer(Y.type),
 ### Panic Branislav.
     Rule = as.character(model@Rule),
 ### End    
@@ -140,7 +152,7 @@ function(model, ...)
     PACKAGE = "rebmix")
 
   if (output$error == 1) {
-    stop("in RCLRMIX!", call. = FALSE); return(NA)
+    stop("in RCombineComponentsMIX!", call. = FALSE); return(NA)
   }
 
   model@tau <- matrix(data = output$tau, ncol = c, byrow = TRUE)
@@ -149,14 +161,14 @@ function(model, ...)
   rownames(model@tau) <- paste(1:n, sep = "")
 
   if (output$c > 1) {
-    model@from <- output$F
-    model@to <- output$T
-    model@EN <- output$EN
-    model@ED <- output$ED
+    model@from <- output$F[-c]
+    model@to <- output$T[-c]
+    model@EN <- output$EN[-c]
+    model@ED <- output$ED[-c]
   }
 
   output <- .C(C_RCLRMIX,
-    n = n,
+    n = as.integer(n),
     X = as.double(dataset),
     d = as.integer(d),
     c = as.integer(unlist(c)),
@@ -177,40 +189,6 @@ function(model, ...)
 
   model@c <- length(unique.Z)
 
-  i <- length(model@from)
-
-  while (i > 0) {
-    from.in.unique.Z <- model@from[i] %in% unique.Z
-    to.in.unique.Z <- model@to[i] %in% unique.Z
-
-    if (from.in.unique.Z && to.in.unique.Z) {
-    }
-    else
-    if (from.in.unique.Z) {
-      j <- 1
-
-      while (j < i) {
-        if (model@from[j] == model@to[i]) model@from[j] <- model@from[i]
-        if (model@to[j] == model@to[i]) model@to[j] <- model@from[i]
-
-        j <- j + 1
-      }
-
-      model@from <- model@from[-i]
-      model@to <- model@to[-i]
-      model@EN <- model@EN[-i]
-      model@ED <- model@ED[-i]
-    }
-    else {
-      model@from <- model@from[-i]
-      model@to <- model@to[-i]
-      model@EN <- model@EN[-i]
-      model@ED <- model@ED[-i]
-    }
-
-    i <- i - 1
-  }
-
   model@Zp <- as.factor(output$Z)
   
   if (model@c < c) {
@@ -247,13 +225,24 @@ function(model, ...)
   dataset <- model@Dataset
 
   if (missing(dataset) || (length(dataset) == 0)) {
-    dataset <- as.matrix(model@x@Dataset[[model@pos]])
+    dataset <- model@x@Dataset[[model@pos]]
   }
-  else {
+    
+  if (as.character(class(dataset)) == "data.frame") {
+    Y.type <- 0
+    
     dataset <- as.matrix(dataset)
-  } 
-
-  n <- nrow(dataset)
+    
+    n <- nrow(dataset)
+  }  
+  else
+  if (as.character(class(dataset)) == "Histogram") {
+    Y.type <- 1
+    
+    dataset <- as.matrix(dataset@Y)
+    
+    n <- nrow(dataset)    
+  }
 
   output <- .C(C_RCombineComponentsMVNORM,
     c = as.integer(c),
@@ -264,7 +253,8 @@ function(model, ...)
     pdf = as.character(pdf),
     Theta = as.double(c(theta1, theta2)),
     n = as.integer(n),
-    x = as.double(dataset),
+    Y = as.double(dataset),
+    Y.type = as.integer(Y.type),
 ### Panic Branislav.
     Rule = as.character(model@Rule),
 ### End    
@@ -277,7 +267,7 @@ function(model, ...)
     PACKAGE = "rebmix")
 
   if (output$error == 1) {
-    stop("in RCLRMIX!", call. = FALSE); return(NA)
+    stop("in RCombineComponentsMVNORM!", call. = FALSE); return(NA)
   }
 
   model@tau <- matrix(data = output$tau, ncol = c, byrow = TRUE)
@@ -286,14 +276,14 @@ function(model, ...)
   rownames(model@tau) <- paste(1:n, sep = "")
 
   if (output$c > 1) {
-    model@from <- output$F
-    model@to <- output$T
-    model@EN <- output$EN
-    model@ED <- output$ED
+    model@from <- output$F[-c]
+    model@to <- output$T[-c]
+    model@EN <- output$EN[-c]
+    model@ED <- output$ED[-c]
   }
 
   output <- .C(C_RCLRMVNORM,
-    n = n,
+    n = as.integer(n),
     X = as.double(dataset),
     d = as.integer(d),
     c = as.integer(unlist(c)),
@@ -306,47 +296,13 @@ function(model, ...)
     PACKAGE = "rebmix")
 
   if (output$error == 1) {
-    stop("in RCLRMIX!", call. = FALSE); return(NA)
+    stop("in RCLRMVNORM!", call. = FALSE); return(NA)
   }
 
   unique.Z <- unique(output$Z)
 
   model@c <- length(unique.Z)
 
-  i <- length(model@from)
-
-  while (i > 0) {
-    from.in.unique.Z <- model@from[i] %in% unique.Z
-    to.in.unique.Z <- model@to[i] %in% unique.Z
-
-    if (from.in.unique.Z && to.in.unique.Z) {
-    }
-    else
-    if (from.in.unique.Z) {
-      j <- 1
-
-      while (j < i) {
-        if (model@from[j] == model@to[i]) model@from[j] <- model@from[i]
-        if (model@to[j] == model@to[i]) model@to[j] <- model@from[i]
-
-        j <- j + 1
-      }
-
-      model@from <- model@from[-i]
-      model@to <- model@to[-i]
-      model@EN <- model@EN[-i]
-      model@ED <- model@ED[-i]
-    }
-    else {
-      model@from <- model@from[-i]
-      model@to <- model@to[-i]
-      model@EN <- model@EN[-i]
-      model@ED <- model@ED[-i]
-    }
-
-    i <- i - 1
-  }
-
   model@Zp <- as.factor(output$Z)
   
   if (model@c < c) {
@@ -369,7 +325,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("RCLRMIX Version 2.14.0")
+  message("RCLRMIX Version 2.14.1")
 
   flush.console()
 
