@@ -100,76 +100,89 @@ function(x,
     pty = pty,
     tcl = tcl, ...)
 
+  item <- list(); items <- NULL
+
+  if (!is.na(x@summary[pos, "Dataset"])) {
+    item[[1]] <- "Dataset"
+    item[[2]] <- " = "
+    item[[3]] <- paste(x@summary[pos, "Dataset"], ", ", sep = "")
+    
+    items <- c(items, 2:3)
+  }
+
+  if (!is.na(x@summary[pos, "Preprocessing"])) {
+    item[[4]] <- "Preprocessing"
+    item[[5]] <- " = "
+    item[[6]] <- paste(x@summary[pos, "Preprocessing"], ", ", sep = "")
+    
+    items <- c(items, 4:6)
+  }
+
+  if (!is.na(x@summary[pos, "cmax"])) {
+    item[[7]] <- bquote(c[max])
+    item[[8]] <- " = "
+    item[[9]] <- paste(x@summary[pos, "cmax"], ", ", sep = "")
+    
+    items <- c(items, 7:9)
+  }
+  
+  if (!is.na(x@summary[pos, "c"])) {
+    item[[10]] <- "c"
+    item[[11]] <- " = "
+    item[[12]] <- paste(x@summary[pos, "c"], ", ", sep = "")
+    
+    items <- c(items, 10:12)
+  }
+  
   C <- x@summary[pos, "Preprocessing"]
 
-  if (is.na(C)) {
-    C <- "NA"
+  if (!is.na(C)) {
+    if (C == .rebmix$Preprocessing[1]) {
+      if (!is.na(x@summary[pos, "v/k"])) {
+        item[[13]] <- "v"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }
+    }
+    else
+    if (C == .rebmix$Preprocessing[2]) {
+      if (!is.na(x@summary[pos, "v/k"])) {    
+        item[[13]] <- "v"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }        
+    }
+    else
+    if (C == .rebmix$Preprocessing[3]) {
+      if (!is.na(x@summary[pos, "v/k"])) {      
+        item[[13]] <- "k"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }          
+    }
+  }
+  
+  if (!is.na(x@summary[pos, "IC"])) {  
+    item[[16]] <- as.character(x@summary[pos, "Criterion"])
+    item[[17]] <- " = "
+    item[[18]] <- paste(as.number(x@summary[pos, "IC"]), ", ", sep = "")
     
-    items <- c(2:3, 19:21, 31:33)
-  }
-  else {
-    items <- c(2:6, 13:21, 25:33)
+    items <- c(items, 16:18)    
   }
 
-  item <- list()
-
-  item[[1]] <- "Dataset"
-  item[[2]] <- " = "
-  item[[3]] <- paste(x@summary[pos, "Dataset"], ", ", sep = "")
-
-  item[[4]] <- "Preprocessing"
-  item[[5]] <- " = "
-  item[[6]] <- paste(x@summary[pos, "Preprocessing"], ", ", sep = "")
-
-  item[[7]] <- ""
-  item[[8]] <- ""
-  item[[9]] <- ""
-
-  item[[10]] <- ""
-  item[[11]] <- ""
-  item[[12]] <- ""
-
-  item[[13]] <- bquote(c[max])
-  item[[14]] <- " = "
-  item[[15]] <- paste(x@summary[pos, "cmax"], ", ", sep = "")
-
-  item[[16]] <- bquote(a[r])
-  item[[17]] <- " = "
-  item[[18]] <- paste(as.number(x@summary[pos, "ar"]), ", ", sep = "")
-
-  item[[19]] <- "c"
-  item[[20]] <- " = "
-  item[[21]] <- paste(x@summary[pos, "c"], ", ", sep = "")
-
-  item[[22]] <- ""
-  item[[23]] <- ""
-  item[[24]] <- ""
-
-  if (C == .rebmix$Preprocessing[1]) {
-    item[[25]] <- "v"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+  if (!is.na(x@summary[pos, "logL"])) { 
+    item[[19]] <- "log L"
+    item[[20]] <- " = "
+    item[[21]] <- paste(as.number(x@summary[pos, "logL"]), ".", sep = "")
+    
+    items <- c(items, 19:21)   
   }
-  else
-  if (C == .rebmix$Preprocessing[2]) {
-    item[[25]] <- "v"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
-  }
-  else
-  if (C == .rebmix$Preprocessing[3]) {
-    item[[25]] <- "k"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
-  }
-
-  item[[28]] <- as.character(x@summary[pos, "Criterion"])
-  item[[29]] <- " = "
-  item[[30]] <- paste(as.number(x@summary[pos, "IC"]), ", ", sep = "")
-
-  item[[31]] <- "log L"
-  item[[32]] <- " = "
-  item[[33]] <- paste(as.number(x@summary[pos, "logL"]), ".", sep = "")
 
   i <- 1; legend <- list(); legend[[i]] <- bquote(.(item[[1]]))
 
@@ -185,10 +198,20 @@ function(x,
   }
 
   par(oma = c(length(legend) + 0.2, 0.2, 0.2, 0.2))
-
-  Dataset <- as.character(x@summary[pos, "Dataset"])
-
-  ey <- as.matrix(x@Dataset[[which(names(x@Dataset) == x@summary[pos, "Dataset"])]])
+  
+  Dataset <- x@Dataset[[which(names(x@Dataset) == x@summary[pos, "Dataset"])]]
+  
+  if (as.character(class(Dataset)) == "data.frame") {
+    Y.type <- 0
+    
+    ey <- as.matrix(Dataset)
+  }
+  else
+  if (as.character(class(Dataset)) == "Histogram") {
+    Y.type <- 1
+    
+    ey <- as.matrix(Dataset@Y)
+  }
 
   y0 <- array(data = 0.0, dim = d, dimnames = NULL)
   h <- array(data = 0.0, dim = d, dimnames = NULL)
@@ -202,32 +225,43 @@ function(x,
   pdf <- match.arg(x@pdf, .rebmix$pdf, several.ok = TRUE)
 
   for (i in 1:d) {
-    if (C == .rebmix$Preprocessing[1]) {
-      k <- as.numeric(x@summary[pos, "v/k"])
-      y0[i] <- as.numeric(x@summary[pos, paste("y0", if (d > 1) i, sep = "")])
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+    if (Y.type == 0) {
+      if (is.na(C)) {
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
+      else    
+      if (C == .rebmix$Preprocessing[1]) {
+        k <- as.numeric(x@summary[pos, "v/k"])
+        y0[i] <- as.numeric(x@summary[pos, paste("y0", if (d > 1) i, sep = "")])
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
 
-      lim[, i] <- range(ey[, i], finite = TRUE)
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
+      else
+      if (C == .rebmix$Preprocessing[2]) {
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
+      else
+      if (C == .rebmix$Preprocessing[3]) {
+        k <- as.numeric(x@summary[pos, "v/k"])
+
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
     }
     else
-    if (C == .rebmix$Preprocessing[2]) {
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
-
-      lim[, i] <- range(ey[, i], finite = TRUE)
-    }
-    else
-    if (C == .rebmix$Preprocessing[3]) {
-      k <- as.numeric(x@summary[pos, "v/k"])
-
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
-
-      lim[, i] <- range(ey[, i], finite = TRUE)
-    }
-    else {
+    if (Y.type == 1) {
+      v <- nrow(ey)
+      
+      h[i] <- Dataset@h[i]
+    
       lim[, i] <- range(ey[, i], finite = TRUE)
     }
 
-    if (abs(lim[2, i] - lim[1, i]) < 1e-6) {
+    if (abs(lim[2, i] - lim[1, i]) < 1.E-6) {
       lim[2, i] <- lim[1, i] + 1.0
     }
 
@@ -257,22 +291,36 @@ function(x,
           
           zlim <- range(pdens, finite = TRUE)
           
-          if (C == .rebmix$Preprocessing[1]) {
-            edens <- .densHistogram.xy(k, ey[, i], ey[, j], y0[i], lim[, i][1], lim[, i][2], y0[j], lim[, j][1], lim[, j][2], h[i], h[j], Variables[i], Variables[j], pdf[i], pdf[j])
+          if (Y.type == 0) {
+            if (is.na(C)) {
+              edens <- .densSample.xy(ey[, i], ey[, j], zlim[1], n)
+            }
+            else          
+            if (C == .rebmix$Preprocessing[1]) {
+              edens <- .densHistogram.xy(k, ey[, i], ey[, j], y0[i], lim[, i][1], lim[, i][2], y0[j], lim[, j][1], lim[, j][2], h[i], h[j], Variables[i], Variables[j], pdf[i], pdf[j])
+            }
+            else
+            if (C == .rebmix$Preprocessing[2]) {
+              edens <- .densKDE.xy(ey[, i], ey[, j], h[i], h[j], n)
+            }
+            else
+            if (C == .rebmix$Preprocessing[3]) {
+              edens <- .densKNearestNeighbour.xy(ey[, i], ey[, j], k, h[i], h[j], n)
+            }
           }
           else
-          if (C == .rebmix$Preprocessing[2]) {
-            edens <- .densKDE.xy(ey[, i], ey[, j], h[i], h[j], n)
-          }
-          else
-          if (C == .rebmix$Preprocessing[3]) {
-            edens <- .densKNearestNeighbour.xy(ey[, i], ey[, j], k, h[i], h[j], n)
-          }
-          else {
-            edens <- .densSample.xy(ey[, i], ey[, j], zlim[1], n)
+          if (Y.type == 1) {
+            edens <- .densK.xy(v, ey[, i], ey[, j], ey[, d + 1], h[i], h[j])
           }
 
           zlim <- range(zlim, edens$z, finite = TRUE)
+          
+          if (zlim[1] < 1.E-100) zlim[1] <- 1.E-100
+          if (zlim[2] < 2.E-100) zlim[2] <- 2.E-100
+          
+          coln <- (edens$z - zlim[1]) / (zlim[2] - zlim[1])
+          
+          coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)          
 
           plot(x = edens$x,
             y = edens$y,
@@ -281,7 +329,7 @@ function(x,
             sub = "",
             xlab = "",
             ylab = "",
-            col = rgb(ramp((edens$z - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+            col = rgb(ramp(coln), maxColorValue = 255),
             axes = FALSE,
             lwd = 1,
             cex = plot.cex,
@@ -290,13 +338,17 @@ function(x,
 
           if ((Variables[i] == .rebmix$Variables[2]) && (Variables[j] == .rebmix$Variables[2])) {
             z <- as.vector(pdens); z <- z != 0.0
+            
+            coln <- (pdens[z] - zlim[1]) / (zlim[2] - zlim[1])
+          
+            coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)              
 
             points(x = rep(py[[i]], length(py[[j]]))[z],
               y = rep(py[[j]], each = length(py[[i]]))[z],
               type = "p",
               xlab = "",
               ylab = "",
-              col = rgb(ramp((pdens[z] - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+              col = rgb(ramp(coln), maxColorValue = 255),
               lwd = 1,
               cex = plot.cex * 0.5,
               pch = plot.pch)
@@ -308,6 +360,10 @@ function(x,
               ty <- py[[j]]
 
               s <- 1:(length(tx) - 1)
+              
+              coln <- ((pdens[l, s] + pdens[l, s + 1]) / 2.0 - zlim[1]) / (zlim[2] - zlim[1])
+          
+              coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)                 
 
               segments(x0 = tx[s],
                 y0 = ty[s],
@@ -315,7 +371,7 @@ function(x,
                 y1 = ty[s + 1],
                 xlab = "",
                 ylab = "",
-                col = rgb(ramp(((pdens[l, s] + pdens[l, s + 1]) / 2.0 - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+                col = rgb(ramp(coln), maxColorValue = 255),
                 cex = plot.cex)
             }
           }
@@ -326,6 +382,10 @@ function(x,
               ty <- rep(py[[j]][l], length(py[[i]]))
 
               s <- 1:(length(tx) - 1)
+              
+              coln <- ((pdens[s, l] + pdens[s + 1, l]) / 2.0 - zlim[1]) / (zlim[2] - zlim[1])
+          
+              coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)                 
 
               segments(x0 = tx[s],
                 y0 = ty[s],
@@ -333,12 +393,16 @@ function(x,
                 y1 = ty[s + 1],
                 xlab = "",
                 ylab = "",
-                col = rgb(ramp(((pdens[s, l] + pdens[s + 1, l]) / 2.0 - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+                col = rgb(ramp(coln), maxColorValue = 255),
                 cex = plot.cex)
             }
           }
           else {
-            levels <- seq(from = zlim[1], to = zlim[2], length.out = contour.nlevels)
+            levels <- exp(seq(from = log(zlim[1]), to = log(zlim[2]), length.out = contour.nlevels))    
+          
+            coln <- (levels - zlim[1]) / (zlim[2] - zlim[1])
+          
+            coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)             
 
             contour(x = py[[i]],
               y = py[[j]],
@@ -349,7 +413,7 @@ function(x,
               zlim = zlim,
               labcex = contour.labcex, drawlabels = contour.drawlabels, method = contour.method,
               axes = FALSE, frame.plot = FALSE,
-              col = rgb(ramp((levels - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+              col = rgb(ramp(coln), maxColorValue = 255),
               add = TRUE)
           }
 
@@ -409,19 +473,26 @@ function(x,
     
       ylim <- range(pdens, finite = TRUE)
     
-      if (C == .rebmix$Preprocessing[1]) {
-        edens <- .densHistogram.x(k, ey[, i], y0[i], lim[, i][1], lim[, i][2], h[i], Variables[i], pdf[i])
+      if (Y.type == 0) { 
+        if (is.na(C)) {
+          edens <- .densSample.x(ey[, i], ylim[1], n)        
+        }
+        else
+        if (C == .rebmix$Preprocessing[1]) {
+          edens <- .densHistogram.x(k, ey[, i], y0[i], lim[, i][1], lim[, i][2], h[i], Variables[i], pdf[i])
+        }
+        else
+        if (C == .rebmix$Preprocessing[2]) {
+          edens <- .densKDE.x(ey[, i], h[i], n)
+        }
+        else
+        if (C == .rebmix$Preprocessing[3]) {
+          edens <- .densKNearestNeighbour.x(ey[, i], k, h[i], n)
+        }
       }
       else
-      if (C == .rebmix$Preprocessing[2]) {
-        edens <- .densKDE.x(ey[, i], h[i], n)
-      }
-      else
-      if (C == .rebmix$Preprocessing[3]) {
-        edens <- .densKNearestNeighbour.x(ey[, i], k, h[i], n)
-      }
-      else {
-        edens <- .densSample.x(ey[, i], ylim[1], n)
+      if (Y.type == 1) {
+        edens <- .densK.x(v, ey[, i], ey[, d + 1], h[i])
       }
 
       ylim <- range(ylim, edens$y, finite = TRUE)
@@ -497,7 +568,13 @@ function(x,
     for (i in 1:d) {
       pdist <- .pfmix.x(py[[i]], w, Theta[i,])
           
-      edist <- .dist.x(ey[, i], n)
+      if (Y.type == 0) {
+        edist <- .dist.x(ey[, i], NULL, n)
+      }
+      else
+      if (Y.type == 1) {
+        edist <- .dist.x(ey[, i], ey[, d + 1], n)
+      }  
 
       ylim <- range(pdist, edist$y, finite = TRUE)
 
@@ -600,7 +677,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[28]]))
+    text <- bquote(c - .(item[[16]]))
 
     mtext(text = text,
       side = 1,
@@ -663,7 +740,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[31]]))
+    text <- bquote(c - .(item[[19]]))
 
     mtext(text = text,
       side = 1,
@@ -726,7 +803,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[10]]))
+    text <- bquote(c - D)
 
     mtext(text = text,
       side = 1,
@@ -789,7 +866,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(K - .(item[[28]]))
+    text <- bquote(K - .(item[[16]]))
 
     mtext(text = text,
       side = 1,
@@ -926,76 +1003,89 @@ function(x,
     pty = pty,
     tcl = tcl, ...)
 
-  C <- x@summary[pos, "Preprocessing"]
-  
-  if (is.na(C)) {
-    C <- "NA"
+  item <- list(); items <- NULL
+
+  if (!is.na(x@summary[pos, "Dataset"])) {
+    item[[1]] <- "Dataset"
+    item[[2]] <- " = "
+    item[[3]] <- paste(x@summary[pos, "Dataset"], ", ", sep = "")
     
-    items <- c(2:3, 19:21, 31:33)
-  }
-  else {
-    items <- c(2:6, 13:21, 25:33)
+    items <- c(items, 2:3)
   }
 
-  item <- list()
+  if (!is.na(x@summary[pos, "Preprocessing"])) {
+    item[[4]] <- "Preprocessing"
+    item[[5]] <- " = "
+    item[[6]] <- paste(x@summary[pos, "Preprocessing"], ", ", sep = "")
+    
+    items <- c(items, 4:6)
+  }
+
+  if (!is.na(x@summary[pos, "cmax"])) {
+    item[[7]] <- bquote(c[max])
+    item[[8]] <- " = "
+    item[[9]] <- paste(x@summary[pos, "cmax"], ", ", sep = "")
+    
+    items <- c(items, 7:9)
+  }
   
-  item[[1]] <- "Dataset"
-  item[[2]] <- " = "
-  item[[3]] <- paste(x@summary[pos, "Dataset"], ", ", sep = "")
-
-  item[[4]] <- "Preprocessing"
-  item[[5]] <- " = "
-  item[[6]] <- paste(x@summary[pos, "Preprocessing"], ", ", sep = "")
+  if (!is.na(x@summary[pos, "c"])) {
+    item[[10]] <- "c"
+    item[[11]] <- " = "
+    item[[12]] <- paste(x@summary[pos, "c"], ", ", sep = "")
+    
+    items <- c(items, 10:12)
+  }
   
-  item[[7]] <- ""
-  item[[8]] <- ""
-  item[[9]] <- ""
+  C <- x@summary[pos, "Preprocessing"]
 
-  item[[10]] <- ""
-  item[[11]] <- ""
-  item[[12]] <- ""
-
-  item[[13]] <- bquote(c[max])
-  item[[14]] <- " = "
-  item[[15]] <- paste(x@summary[pos, "cmax"], ", ", sep = "")
-
-  item[[16]] <- bquote(a[r])
-  item[[17]] <- " = "
-  item[[18]] <- paste(as.number(x@summary[pos, "ar"]), ", ", sep = "")
-
-  item[[19]] <- "c"
-  item[[20]] <- " = "
-  item[[21]] <- paste(x@summary[pos, "c"], ", ", sep = "")
-
-  item[[22]] <- ""
-  item[[23]] <- ""
-  item[[24]] <- ""
-
-  if (C == .rebmix$Preprocessing[1]) {
-    item[[25]] <- "v"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+  if (!is.na(C)) {
+    if (C == .rebmix$Preprocessing[1]) {
+      if (!is.na(x@summary[pos, "v/k"])) {
+        item[[13]] <- "v"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }
+    }
+    else
+    if (C == .rebmix$Preprocessing[2]) {
+      if (!is.na(x@summary[pos, "v/k"])) {    
+        item[[13]] <- "v"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }        
+    }
+    else
+    if (C == .rebmix$Preprocessing[3]) {
+      if (!is.na(x@summary[pos, "v/k"])) {      
+        item[[13]] <- "k"
+        item[[14]] <- " = "
+        item[[15]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+        
+        items <- c(items, 13:15)
+      }          
+    }
   }
-  else
-  if (C == .rebmix$Preprocessing[2]) {
-    item[[25]] <- "v"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
-  }
-  else
-  if (C == .rebmix$Preprocessing[3]) {
-    item[[25]] <- "k"
-    item[[26]] <- " = "
-    item[[27]] <- paste(x@summary[pos, "v/k"], ", ", sep = "")
+  
+  if (!is.na(x@summary[pos, "IC"])) {  
+    item[[16]] <- as.character(x@summary[pos, "Criterion"])
+    item[[17]] <- " = "
+    item[[18]] <- paste(as.number(x@summary[pos, "IC"]), ", ", sep = "")
+    
+    items <- c(items, 16:18)    
   }
 
-  item[[28]] <- as.character(x@summary[pos, "Criterion"])
-  item[[29]] <- " = "
-  item[[30]] <- paste(as.number(x@summary[pos, "IC"]), ", ", sep = "")
-
-  item[[31]] <- "log L"
-  item[[32]] <- " = "
-  item[[33]] <- paste(as.number(x@summary[pos, "logL"]), ".", sep = "")
+  if (!is.na(x@summary[pos, "logL"])) { 
+    item[[19]] <- "log L"
+    item[[20]] <- " = "
+    item[[21]] <- paste(as.number(x@summary[pos, "logL"]), ".", sep = "")
+    
+    items <- c(items, 19:21)   
+  }
 
   i <- 1; legend <- list(); legend[[i]] <- bquote(.(item[[1]]))
 
@@ -1012,9 +1102,19 @@ function(x,
 
   par(oma = c(length(legend) + 0.2, 0.2, 0.2, 0.2))
 
-  Dataset <- as.character(x@summary[pos, "Dataset"])
-
-  ey <- as.matrix(x@Dataset[[which(names(x@Dataset) == x@summary[pos, "Dataset"])]])
+  Dataset <- x@Dataset[[which(names(x@Dataset) == x@summary[pos, "Dataset"])]]
+  
+  if (as.character(class(Dataset)) == "data.frame") {
+    Y.type <- 0
+    
+    ey <- as.matrix(Dataset)
+  }
+  else
+  if (as.character(class(Dataset)) == "Histogram") {
+    Y.type <- 1
+    
+    ey <- as.matrix(Dataset@Y)
+  }
 
   y0 <- array(data = 0.0, dim = d, dimnames = NULL)
   h <- array(data = 0.0, dim = d, dimnames = NULL)
@@ -1028,32 +1128,43 @@ function(x,
   pdf <- match.arg(x@pdf, .rebmix$pdf, several.ok = TRUE)
 
   for (i in 1:d) {
-    if (C == .rebmix$Preprocessing[1]) {
-      k <- as.numeric(x@summary[pos, "v/k"])
-      y0[i] <- as.numeric(x@summary[pos, paste("y0", if (d > 1) i, sep = "")])
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+    if (Y.type == 0) {
+      if (is.na(C)) {
+        lim[, i] <- range(ey[, i], finite = TRUE)      
+      } 
+      else
+      if (C == .rebmix$Preprocessing[1]) {
+        k <- as.numeric(x@summary[pos, "v/k"])
+        y0[i] <- as.numeric(x@summary[pos, paste("y0", if (d > 1) i, sep = "")])
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
 
-      lim[, i] <- range(ey[, i], finite = TRUE)
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
+      else
+      if (C == .rebmix$Preprocessing[2]) {
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
+      else
+      if (C == .rebmix$Preprocessing[3]) {
+        k <- as.numeric(x@summary[pos, "v/k"])
+
+        h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
+
+        lim[, i] <- range(ey[, i], finite = TRUE)
+      }
     }
     else
-    if (C == .rebmix$Preprocessing[2]) {
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
-
-      lim[, i] <- range(ey[, i], finite = TRUE)
-    }
-    else
-    if (C == .rebmix$Preprocessing[3]) {
-      k <- as.numeric(x@summary[pos, "v/k"])
-
-      h[i] <- as.numeric(x@summary[pos, paste("h", if (d > 1) i, sep = "")])
-
-      lim[, i] <- range(ey[, i], finite = TRUE)
-    }
-    else {
+    if (Y.type == 1) {
+      v <- nrow(ey)
+      
+      h[i] <- Dataset@h[i]
+    
       lim[, i] <- range(ey[, i], finite = TRUE)
     }
 
-    if (abs(lim[2, i] - lim[1, i]) < 1e-6) {
+    if (abs(lim[2, i] - lim[1, i]) < 1.E-6) {
       lim[2, i] <- lim[1, i] + 1.0
     }
 
@@ -1076,24 +1187,38 @@ function(x,
         for (j in (i + 1):d) {
           pdens <- outer(py[[i]], py[[j]], ".dfmvnorm.xy", w, Theta, i, j)
           
-          zlim <- range(pdens, finite = TRUE);       
-        
-          if (C == .rebmix$Preprocessing[1]) {
-            edens <- .densHistogram.xy(k, ey[, i], ey[, j], y0[i], lim[, i][1], lim[, i][2], y0[j], lim[, j][1], lim[, j][2], h[i], h[j], Variables[i], Variables[j], pdf[i], pdf[j])
+          zlim <- range(pdens, finite = TRUE);
+          
+          if (Y.type == 0) {   
+            if (is.na(C)) {
+              edens <- .densSample.xy(ey[, i], ey[, j], zlim[1], n)    
+            }
+            else            
+            if (C == .rebmix$Preprocessing[1]) {
+              edens <- .densHistogram.xy(k, ey[, i], ey[, j], y0[i], lim[, i][1], lim[, i][2], y0[j], lim[, j][1], lim[, j][2], h[i], h[j], Variables[i], Variables[j], pdf[i], pdf[j])
+            }
+            else
+            if (C == .rebmix$Preprocessing[2]) {
+              edens <- .densKDE.xy(ey[, i], ey[, j], h[i], h[j], n)
+            }
+            else
+            if (C == .rebmix$Preprocessing[3]) {
+              edens <- .densKNearestNeighbour.xy(ey[, i], ey[, j], k, h[i], h[j], n)
+            }
           }
           else
-          if (C == .rebmix$Preprocessing[2]) {
-            edens <- .densKDE.xy(ey[, i], ey[, j], h[i], h[j], n)
-          }
-          else
-          if (C == .rebmix$Preprocessing[3]) {
-            edens <- .densKNearestNeighbour.xy(ey[, i], ey[, j], k, h[i], h[j], n)
-          }
-          else {
-            edens <- .densSample.xy(ey[, i], ey[, j], zlim[1], n)
-          }
+          if (Y.type == 1) {
+            edens <- .densK.xy(v, ey[, i], ey[, j], ey[, d + 1], h[i], h[j])
+          }        
           
           zlim <- range(zlim, edens$z, finite = TRUE)
+          
+          if (zlim[1] < 1.E-100) zlim[1] <- 1.E-100
+          if (zlim[2] < 2.E-100) zlim[2] <- 2.E-100          
+          
+          coln <- (edens$z - zlim[1]) / (zlim[2] - zlim[1])
+          
+          coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)
 
           plot(x = edens$x,
             y = edens$y,
@@ -1102,14 +1227,18 @@ function(x,
             sub = "",
             xlab = "",
             ylab = "",
-            col = rgb(ramp((edens$z - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+            col = rgb(ramp(coln), maxColorValue = 255),
             axes = FALSE,
             lwd = 1,
             cex = plot.cex,
             pch = plot.pch, 
             log = log)
-
-          levels <- seq(from = zlim[1], to = zlim[2], length.out = contour.nlevels)
+            
+          levels <- exp(seq(from = log(zlim[1]), to = log(zlim[2]), length.out = contour.nlevels))    
+          
+          coln <- (levels - zlim[1]) / (zlim[2] - zlim[1])
+          
+          coln <- sapply(coln, function(x) if (x < 0.0) x <- 0.0 else if (x > 1.0) x <- 1.0 else x)               
           
           contour(x = py[[i]],
             y = py[[j]],
@@ -1120,7 +1249,7 @@ function(x,
             zlim = zlim,
             labcex = contour.labcex, drawlabels = contour.drawlabels, method = contour.method,
             axes = FALSE, frame.plot = FALSE,
-            col = rgb(ramp((levels - zlim[1]) / (zlim[2] - zlim[1])), maxColorValue = 255),
+            col = rgb(ramp(coln), maxColorValue = 255),
             add = TRUE)
 
           box(col = fg, lty = "solid", lwd = 1)
@@ -1179,20 +1308,27 @@ function(x,
       
       ylim <- range(pdens, finite = TRUE)
       
-      if (C == .rebmix$Preprocessing[1]) {
-        edens <- .densHistogram.x(k, ey[, i], y0[i], lim[, i][1], lim[, i][2], h[i], Variables[i], pdf[i])
+      if (Y.type == 0) {  
+        if (is.na(C)) {
+          edens <- .densSample.x(ey[, i], ylim[1], n)   
+        }
+        else             
+        if (C == .rebmix$Preprocessing[1]) {
+          edens <- .densHistogram.x(k, ey[, i], y0[i], lim[, i][1], lim[, i][2], h[i], Variables[i], pdf[i])
+        }
+        else
+        if (C == .rebmix$Preprocessing[2]) {
+          edens <- .densKDE.x(ey[, i], h[i], n)
+        }
+        else
+        if (C == .rebmix$Preprocessing[3]) {
+          edens <- .densKNearestNeighbour.x(ey[, i], k, h[i], n)
+        }
       }
       else
-      if (C == .rebmix$Preprocessing[2]) {
-        edens <- .densKDE.x(ey[, i], h[i], n)
-      }
-      else
-      if (C == .rebmix$Preprocessing[3]) {
-        edens <- .densKNearestNeighbour.x(ey[, i], k, h[i], n)
-      }
-      else {
-        edens <- .densSample.x(ey[, i], ylim[1], n)
-      }
+      if (Y.type == 1) {
+        edens <- .densK.x(v, ey[, i], ey[, d + 1], h[i])      
+      }      
       
       ylim <- range(ylim, edens$y, finite = TRUE)
 
@@ -1267,7 +1403,13 @@ function(x,
     for (i in 1:d) {
       pdist <- .pfmvnorm.x(py[[i]], w, Theta, i)
     
-      edist <- .dist.x(ey[, i], n)
+      if (Y.type == 0) {
+        edist <- .dist.x(ey[, i], NULL, n)
+      }
+      else
+      if (Y.type == 1) {
+        edist <- .dist.x(ey[, i], ey[, d + 1], n)
+      }
 
       ylim <- range(pdist, edist$y, finite = TRUE)    
       
@@ -1370,7 +1512,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[28]]))
+    text <- bquote(c - .(item[[16]]))
 
     mtext(text = text,
       side = 1,
@@ -1433,7 +1575,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[31]]))
+    text <- bquote(c - .(item[[19]]))
 
     mtext(text = text,
       side = 1,
@@ -1496,7 +1638,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(c - .(item[[10]]))
+    text <- bquote(c - D)
 
     mtext(text = text,
       side = 1,
@@ -1559,7 +1701,7 @@ function(x,
       hadj = 0.5,
       padj = 1.0)
 
-    text <- bquote(K - .(item[[28]]))
+    text <- bquote(K - .(item[[16]]))
 
     mtext(text = text,
       side = 1,
