@@ -13,11 +13,11 @@ options(prompt = "> ", continue = "+ ", width = 70,
 ## Weibullnormal dataset ##
 ###########################
 
-data("weibullnormal", package = "rebmix")
+data(weibullnormal, package = "rebmix")
 
 ########## FlexMix ##########
 
-library("flexmix")
+library(flexmix)
 
 set.seed(8)
 
@@ -71,11 +71,11 @@ WNmodel <- function (formula = .~.) {
 
       beta <- beta - dbeta
 
-      if (is.nan(dbeta) || is.infinite(dbeta) || (beta <= 1e-6)) {
+      if (is.nan(dbeta) || is.infinite(dbeta) || (beta <= 1.E-6)) {
         stop("Not converged!")
       }
 
-      if (abs(dbeta / beta) < 1e-6) {
+      if (abs(dbeta / beta) < 1.E-6) {
         Error <- 0
       }
 
@@ -105,7 +105,7 @@ summary(weibullnormalestFlexMix)
 
 ########## REBMIX ##########
 
-library("rebmix")
+library(rebmix)
 
 # Estimate number of components, component weights and component parameters.
 
